/**
 * Copyright (c) 2007, Ben L. Titzer
 * See the file "license.txt" for details.
 *
 * Created Jan 27, 2008
 */
package vpc.core.virgil;

import vpc.core.base.Function;
import vpc.core.base.Tuple;
import vpc.core.types.Type;

/**
 * The <code>V3TypeSystem</code> definition.
 *
 * @author Ben L. Titzer
 */
public class V3TypeSystem extends V2TypeSystem {

    protected boolean isSubtypeFunction(Type src, Type target) {
        if (isNull(src)) return true;
        if (!isFunction(target)) return false;
        Function.IType fsrc = (Function.IType)src;
        Function.IType ftarget = (Function.IType)target;
        Type[] srcParamTypes = Tuple.getParameterTypes(fsrc);
        Type[] targetParamTypes = Tuple.getParameterTypes(ftarget);
        if (srcParamTypes.length != targetParamTypes.length) return false;
        for (int i = 0; i < srcParamTypes.length; i++) {
            // functions are contra-variant in the parameter types
            if (!isAssignable(targetParamTypes[i], srcParamTypes[i])) return false;
        }
        // functions are co-variant in the return type
        return isAssignable(fsrc.getResultType(), ftarget.getResultType());
    }

    protected boolean isSubtypeTuple(Type src, Type target) {
        if (!isTuple(target)) return false;
        Tuple.IType fsrc = (Tuple.IType)src;
        Tuple.IType ftarget = (Tuple.IType)target;
        Type[] srcTypes = fsrc.elements();
        Type[] targetTypes = ftarget.elements();
        for (int i = 0; i < srcTypes.length; i++) {
            // tuples are co-variant in the parameter type
            if (!isAssignable(srcTypes[i], targetTypes[i])) return false;
        }
        return true;
    }
}
